/*
 Wireless Gameboy/GBA Controller - Nintendo 64 RX
 Version: 1.2
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 13/03/2019
 Last Modified: 30/05/2019
 
 */

#include <avr/io.h>
#include <avr/interrupt.h>

extern uint8_t firstPacket;
extern uint8_t kpData;
extern uint8_t kpDataHigh;
extern uint8_t kpOutputDpad;
extern uint8_t kpOutputMapYB;
extern uint8_t kpOutputMapLtoZ;
extern uint8_t tempKpOutput[4];

void tx_snes(void) {
	// The first packet received triggers special modes
	if (firstPacket == 1) {
		firstPacket = 0;
		
		// Down held down, output the key presses to the D-Pad
		if (multiTxData[1] & 0x04) {
			kpOutputDpad = 1;
		}
		// A held down, map the SNES Y button to the N64 B button and SNES B button to the N64 A button
		if (multiTxData[2] & 0x80) { // A
			kpOutputMapYB = 1;
		}
		// L held down, map the SNES L bytton to the N64 Z button
		if (multiTxData[2] & 0x20) {
			kpOutputMapLtoZ = 1;
		}
	}
	
	if (multiTxData[1] & 0x80) { // B
		if (kpOutputMapYB == 1) {
			tempKpOutput[0] |= 0x80;
		}
		else {
			tempKpOutput[0] |= 0x40;
		}
	}
	if (multiTxData[1] & 0x40) { // Y
		if (kpOutputMapYB == 1) {
			tempKpOutput[0] |= 0x40;
		}
	}
	if (multiTxData[1] & 0x20) { // Select
		tempKpOutput[0] |= 0x20; // Z
	}
	if (multiTxData[1] & 0x10) { // Start
		tempKpOutput[0] |= 0x10;
	}
	if (multiTxData[1] & 0x08) { // Up
		if (kpOutputDpad == 1) {
			tempKpOutput[0] |= 0x08;
		}
		else {
			tempKpOutput[3] = 80;
		}
	}
	if (multiTxData[1] & 0x04) { // Down
		if (kpOutputDpad == 1) {
			tempKpOutput[0] |= 0x04;
		}
		else {
			tempKpOutput[3] = -80;
		}
	}
	if (multiTxData[1] & 0x02) { // Left
		if (kpOutputDpad == 1) {
			tempKpOutput[0] |= 0x02;
		}
		else {
			tempKpOutput[2] = -80;
		}
	}
	if (multiTxData[1] & 0x01) { // Right
		if (kpOutputDpad == 1) {
			tempKpOutput[0] |= 0x01;
		}
		else {
			tempKpOutput[2] = 80;
		}
	}
	if (multiTxData[2] & 0x80) { // A
		tempKpOutput[0] |= 0x80;
	}
	if (multiTxData[2] & 0x40) { // X
		
	}
	if (multiTxData[2] & 0x20) { // L
		if (kpOutputMapLtoZ == 1) {
			tempKpOutput[0] |= 0x20; // Z
		}
		else {
			tempKpOutput[1] |= 0x20;
		}
	}
	if (multiTxData[2] & 0x10) { // R
		tempKpOutput[1] |= 0x10;
	}
}

void tx_gb_gba(void) {
	// The first packet received triggers special modes
	if (firstPacket == 1) {
		firstPacket = 0;
		
		// Down held down, output the key presses to the D-Pad
		if (kpData & 0x20) {
			kpOutputDpad = 1;
		}
		// L held down, map the GBA L button to the N64 Z button
		if (kpDataHigh & 0x01) {
			kpOutputMapLtoZ = 1;
		}
	}

	if (kpData & 0x01) { // A
		tempKpOutput[0] |= 0x80;
	}
	if (kpData & 0x02) { // B
		tempKpOutput[0] |= 0x40;
	}
	if (kpData & 0x04) { // Right
		if (kpOutputDpad == 1) {
			tempKpOutput[0] |= 0x01;
		}
		else {
			tempKpOutput[2] = 80;
		}
	}
	if (kpData & 0x08) { // Left
		if (kpOutputDpad == 1) {
			tempKpOutput[0] |= 0x02;
		}
		else {
			tempKpOutput[2] = -80;
		}
	}
	if (kpData & 0x10) { // Up
		if (kpOutputDpad == 1) {
			tempKpOutput[0] |= 0x08;
		}
		else {
			tempKpOutput[3] = 80;
		}
	}
	if (kpData & 0x20) { // Down
		if (kpOutputDpad == 1) {
			tempKpOutput[0] |= 0x04;
		}
		else {
			tempKpOutput[3] = -80;
		}
	}
	if (kpData & 0x40) { // Select
		tempKpOutput[0] |= 0x20; // Z
	}
	if (kpData & 0x80) { // Start
		tempKpOutput[0] |= 0x10;
	}
	if (kpDataHigh & 0x01) { // L (for GBA)
		if (kpOutputMapLtoZ == 1) {
			tempKpOutput[0] |= 0x20; // Z
		}
		else {
			tempKpOutput[1] |= 0x20;
		}
	}
	if (kpDataHigh & 0x02) { // R (for GBA)
		tempKpOutput[1] |= 0x10;
	}
}
